//+------------------------------------------------------------------+
//|                                                  SOM_Net_Img.mqh |
//|                                            Copyright 2015, Urain |
//|                            https://login.mql5.com/ru/users/Urain |
//+------------------------------------------------------------------+
#property copyright "Copyright 2015, Urain"
#property link      "https://login.mql5.com/ru/users/Urain"
//+------------------------------------------------------------------+
//| #include <SOM_Net_Img.mqh>                                       |
//+------------------------------------------------------------------+
#include "SOM_Net_Train.mqh"
#include "cintbmp2.mqh"
//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
enum ColorSchemes
  {
   Black_White=0,      // -
   Blue_Green_Red=1,   // ----
   Red_Black_Green=2,  // --
   Red_White_Green=3   // --
  };
//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
class CSOM_Net_Img : public CSOM_Net_Train
  {
protected:
   uchar             Palette[768];
   //---      BMP-   
   cIntBMP           m_bmp[];
   int               m_ShiftBmpW;
   int               m_ShiftBmpH;
   //---      
   int               m_gridmode;
   bool              m_HexagonalCell;
   bool              m_ShowBorders;
   bool              m_ShowTitles;
   int               m_ColorScheme;
   int               m_MaxPictures;
   //---    
   void              Blend(uchar c1,uchar c2,uchar r1,uchar g1,uchar b1,uchar r2,uchar g2,uchar b2);
   //---       
   int               GetPalColor(uchar index);
   //---   
   void              RenderCell(int img,int col,int node_index,bool cell_even);
public:
                     CSOM_Net_Img(void){ m_ShiftBmpW=10; m_ShiftBmpH=30;};
                    ~CSOM_Net_Img(void){};
   //---   ,                   
   void              InitParameters(int iterations,int xcells,int ycells,
                                    int bmpwidth,int bmpheight,
                                    bool p_HexagonalCell,bool p_ShowBorders,bool p_ShowTitles,
                                    int p_ColorScheme,int p_MaxPictures);
   //---    
   void              Render();
   //---   bmp-    
   void              ShowBMP(bool back);
   //---     bmp-
   void              SaveBMP();
   //---   ,  BMP-  
   void              NetDeinit();
  };
//+------------------------------------------------------------------+
//|                                         |
//+------------------------------------------------------------------+
void CSOM_Net_Img::InitParameters(int iterations,int xcells,int ycells,
                                  int bmpwidth,int bmpheight,
                                  bool p_HexagonalCell,bool p_ShowBorders,bool p_ShowTitles,int p_ColorScheme,int p_MaxPictures)
  {
//---       
   m_gridmode=int(ChartGetInteger(0,CHART_SHOW_GRID,0));
//---         
   m_HexagonalCell=p_HexagonalCell;
   m_ShowBorders=p_ShowBorders;
   m_ShowTitles=p_ShowTitles;
   m_ColorScheme=p_ColorScheme;
   m_MaxPictures=p_MaxPictures;
//---
   m_xcells=xcells;
   m_ycells=ycells;
//---
   m_xsize=bmpwidth;
   m_ysize=bmpheight;
//---    
   if(m_xsize<m_xcells)m_xsize=m_xcells;
   if(m_ysize<m_ycells)m_ysize=m_ycells;
//---      
   m_iterations=iterations;
   int cellwidth=m_xsize/m_xcells;
   int cellheight=m_ysize/m_ycells;
//---       
   if((double)cellwidth!=m_xsize*1./m_xcells)m_xsize=cellwidth*m_xcells;
   if((double)cellheight!=m_ysize*1./m_ycells)m_ysize=cellheight*m_ycells;
//---  BMP-
   ArrayResize(m_bmp,m_dimension+1);
   for(int i=0; i<m_dimension; i++)
     {
      if(!m_HexagonalCell) {m_bmp[i].Create(m_xsize,m_ysize,clrWhite);}
      else m_bmp[i].Create(m_xsize+cellwidth,m_ysize+cellheight/2,clrWhite);
     }
//---
   switch(m_ColorScheme)
     {
      case 0:
        {
         //--- Black to White gradient
         Blend(0,255,0,0,0,255,255,255);
         break;
        }
      case 1:
        {//--- Blue-green-red
         Blend(0,1*64-1,98,98,255,98,255,255);
         Blend(1*64,2*64-1,98,255,255,98,255,98);
         Blend(2*64,3*64-1,98,255,98,255,255,98);
         Blend(3*64,4*64-1,255,255,98,255,98,98);
         break;
        }
      case 2:
        {//--- Red-black-green
         Blend(0,127,155,0,0,0,0,0);
         Blend(127,255,0,0,0,0,155,0);
         break;
        }
      case 3:
        {//--- Red-white-green
         Blend(0,127,155,0,0,255,255,255);
         Blend(127,255,255,255,255,0,155,0);
         break;
        }
      default:
        {
         //--- Black to White gradient
         Blend(0,255,0,0,0,255,255,255);
         break;
        }
     }
//---        
   ArrayResize(m_som_nodes,xcells*ycells);
//---    
   if(m_xsize>m_ysize) {m_map_radius=m_xsize/2.0;}
   else {m_map_radius=m_ysize/2.0;}
//---
   m_time_constant=1.0*m_iterations/log(m_map_radius);
   Print("m_time_constant=",m_time_constant);
  };
//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
void CSOM_Net_Img::Blend(uchar c1,uchar c2,uchar r1,uchar g1,uchar b1,uchar r2,uchar g2,uchar b2)
  {
   int n=(c2-c1);
   for(int i=0; i<=n; i++)
     {
      if((c1+i+2)<ArraySize(Palette))
        {
         Palette[3*(c1+i)]=uchar(round(1*(r1*(n-i)+r2*i)*1.0/n));
         Palette[3*(c1+i)+1]=uchar (round(1*(g1*(n-i) + g2*i) * 1.0/ n));
         Palette[3*(c1+i)+2]=uchar (round(1*(b1*(n-i) + b2*i) * 1.0/ n));
        }
     }
  };
//+------------------------------------------------------------------+
//|      ( m_bmp)                 |
//+------------------------------------------------------------------+
void CSOM_Net_Img::Render()
  {
   int indx=0;
   for(int i=0; i<m_xcells; i++)
     {
      for(int j=0; j<m_ycells; j++)
        {
         //---        
         for(int k=0; k<m_dimension && !IsStopped(); k++)
           {
            //m_som_nodes[indx].GetCoordinates(x1,y1,x2,y2);
            double range=(m_max_values[k]-m_min_values[k]);
            char avg=char(255*(m_som_nodes[indx].GetWeight(k)-m_min_values[k])/range);
            int col=int(GetPalColor(avg));
            RenderCell(k,col,indx,(j%2==0));
           }
         indx++;
        }
      //---        
      for(int m=0; m<m_dimension; m++)
        {
         for(int k=0; k<15; k++)
           {
            for(int l=0; l<m_xsize; l++)
              {
               uchar colind=uchar(round(l*255.0/m_xsize));
               int col=int(GetPalColor(colind));
               m_bmp[m].DrawDot(l,k,col);
              }
           }
         //---         
         m_bmp[m].TypeText(0,0,DoubleToString(m_min_values[m],2),clrBlack);
         m_bmp[m].TypeText(m_xsize-25,0,DoubleToString(m_max_values[m],2),clrBlack);
         //---
         if(m_ColorScheme==2)
            m_bmp[m].TypeText(m_xsize/2-3*StringLen(m_som_titles[m]),0,m_som_titles[m],clrWhite);
         else
            m_bmp[m].TypeText(m_xsize/2-3*StringLen(m_som_titles[m]),0,m_som_titles[m],clrBlack);
        }
     }
   return;
  };
//+------------------------------------------------------------------+
//|                          |
//+------------------------------------------------------------------+
int CSOM_Net_Img::GetPalColor(uchar index)
  {
   int indx=index;
   if(indx<=0) {indx=0;}
   uchar _r=Palette[3*(indx)];
   uchar _g=Palette[3*(indx)+1];
   uchar _b=Palette[3*(indx)+2];
   int res=_r+256*_g+65536*_b;
//---
   return(res);
  };
//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
void CSOM_Net_Img::RenderCell(int img,int col,int node_index,bool cell_even)
  {
   int x1,y1,x2,y2;
   m_som_nodes[node_index].GetCoordinates(x1,y1,x2,y2);
//---
   if(!m_HexagonalCell)
     {
      //---  
      m_bmp[img].DrawRectangle(x1,y1,x2,y2,col,true);
      if(m_ShowBorders) m_bmp[img].DrawRectangle(x1,y1,x2,y2,clrBlack,false);
     }
   else
     {
      //--- 
      int x_size=int (fabs(x2-x1));
      int y_size=int (fabs(y2-y1));
      if(cell_even)
        {
         x1=x1+x_size/2;
         x2=x2+x_size/2;
        }
      y1=15+y1+y_size/4;
      y2=15+y2+y_size/4;
      //---
      m_bmp[img].DrawRectangle(x1,y1+y_size/4,x2,y2-y_size/4,col,true);
      m_bmp[img].DrawTriangle(x1,y1+y_size/4,x1+x_size/2,y1-y_size/4,x2,y1+y_size/4,col,true);
      m_bmp[img].DrawTriangle(x1,y2-y_size/4,x2,y2-y_size/4,x1+x_size/2,y2+y_size/4,col,true);
      //---
      if(m_ShowBorders)
        {
         //---  
         int bcol=clrBlack;
         m_bmp[img].DrawLine(x1,y1+y_size/4,x1+x_size/2,y1-y_size/4,bcol);
         m_bmp[img].DrawLine(x1+x_size/2,y1-y_size/4,x2,y1+y_size/4,bcol);
         //---
         m_bmp[img].DrawLine(x1,y2-y_size/4,x1+x_size/2,y2+y_size/4,bcol);
         m_bmp[img].DrawLine(x2,y2-y_size/4,x2-x_size/2,y2+y_size/4,bcol);
         //---
         m_bmp[img].DrawLine(x1,y1+y_size/4,x1,y2-y_size/4,bcol);
         m_bmp[img].DrawLine(x2,y1+y_size/4,x2,y2-y_size/4,bcol);
        }
     }
  };
//+------------------------------------------------------------------+
//|   bmp-                 |
//+------------------------------------------------------------------+
void CSOM_Net_Img::ShowBMP(bool back)
  {
   for(int i=0; i<m_dimension; i++)
     {
      //---    BMP-
      m_bmp[i].CreateResource("bmpimg"+IntegerToString(i));
      //---    
      int pos1=i/m_MaxPictures;
      int pos2=i%m_MaxPictures;
      m_bmp[i].ShowResource(m_ShiftBmpW+pos2*(5+m_xsize),m_ShiftBmpH+pos1*(5+m_ysize),"bmpimg"+IntegerToString(i),"SOM"+IntegerToString(i));
      //---    
      ObjectSetInteger(0,"SOM"+IntegerToString(i),OBJPROP_BACK,back);
     }
  };
//+------------------------------------------------------------------+
//|   bmp-                                |
//+------------------------------------------------------------------+
void CSOM_Net_Img::SaveBMP()
  {
   for(int i=0; i<m_dimension; i++)
     {
      //---    BMP-
      m_bmp[i].Save("SOM\\bmpimg"+IntegerToString(i));
     }
  };
//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
void CSOM_Net_Img::NetDeinit()
  {
   Print("Size Node Array = ",ArraySize(m_som_nodes));
//---      
   ChartSetInteger(0,CHART_SHOW_GRID,m_gridmode);
//---   SOM    
   for(int i=0; i<m_dimension; i++)
     {
      m_bmp[i].DeleteResource("bmpimg"+IntegerToString(i));
      m_bmp[i].Hide();
     }
  };
//+------------------------------------------------------------------+
